@@ -13,7 +13,7 @@ from typing import NewType, Union, Optional
 # from System.Collections.Generic import List as CSList
 # from System import Array, Double
 
-from .ParameterStructure.Values import SimMultiValueBigTable, SimMultiValueBigTableHeader
+from .ParameterStructure.Values import SimMultiValueBigTable, SimMultiValueBigTableHeader, SimMultiValuePointer
 from .ParameterStructure.Instances import SimInstancePlacementGeometry, SimInstanceType
 from .ParameterStructure.Parameters import SimParameter
 from .ParameterStructure.Components import SimComponent, SimSlot, SimSlotBase, SimChildComponentEntry, ComponentUtils
@@ -1002,13 +1002,16 @@ class ValueField(pd.DataFrame):
     def __init__(self, *args, **kwargs):
 
         # self._wrapped_obj = kwargs.get('wrapped_obj')
-        self._wrapped_obj = kwargs.pop('wrapped_obj', None)
+        value_field = kwargs.pop('wrapped_obj', None)
+        if isinstance(value_field, SimMultiValuePointer):
+            value_field = value_field.ValueField
+        self._wrapped_obj = value_field
         self._template_parser = kwargs.get('template_parser', None)
         self._data_model_id = kwargs.get('data_model_id', None)
 
-        row_headers = [x.Name for x in self._wrapped_obj.ValueField.RowHeaders.Items]
-        column_headers = [x.Name for x in self._wrapped_obj.ValueField.ColumnHeaders.Items]
-        values = self._wrapped_obj.ValueField.Values
+        row_headers = [x.Name for x in self._wrapped_obj.RowHeaders.Items]
+        column_headers = [x.Name for x in self._wrapped_obj.ColumnHeaders.Items]
+        values = self._wrapped_obj.Values
 
         super().__init__(dict(zip(column_headers, np.array(values).T)),  index=row_headers)
 
