@@ -1,8 +1,5 @@
 from weakref import WeakSet
-import numpy as np
 import colorlog
-# from copy import deepcopy
-# from functools import lru_cache
 from tqdm import tqdm
 from .utils import classproperty
 from .geo_default_types import create_geo_classes
@@ -49,7 +46,10 @@ class GeometryModel(object):
         self._geo_types = kwargs.get('geo_types', None)
 
         self.GeoBaseClass, self.LayerCls, self.VertexCls, self.EdgeCls, self.EdgeLoopCls, self.FaceCls, self.VolumeCls = create_geo_classes(self._geo_types)
-        self.load_all()
+
+        self.loaded = False
+
+        # self.load_all()
 
     @property
     def id(self):
@@ -71,36 +71,49 @@ class GeometryModel(object):
 
     @property
     def layers(self):
+        if not self.loaded:
+            self.load_all()
         if self._layers is None:
             self._layers = self.get_layers()
         return self._layers
 
     @property
     def vertices(self):
+        if not self.loaded:
+            self.load_all()
+
         if self._vertices is None:
             self._vertices = self.get_vertices()
         return self._vertices
 
     @property
     def edges(self):
+        if not self.loaded:
+            self.load_all()
         if self._edges is None:
             self._edges = self.get_edges()
         return self._edges
 
     @property
     def edge_loops(self):
+        if not self.loaded:
+            self.load_all()
         if self._edge_loops is None:
             self._edge_loops = self.get_edge_loops()
         return self._edge_loops
 
     @property
     def faces(self):
+        if not self.loaded:
+            self.load_all()
         if self._faces is None:
             self._faces = self.get_faces()
         return self._faces
 
     @property
     def volumes(self):
+        if not self.loaded:
+            self.load_all()
         if self._volumes is None:
             self._volumes = self.get_volumes()
         return self._volumes
@@ -162,6 +175,8 @@ class GeometryModel(object):
 
     def load_all(self):
 
+        self.loaded = True
+
         logger.info(f'Geometry model: {self.name}: loading vertices')
         _ = self.vertices
         logger.info(f'Geometry model: {self.name}: loading edges')
@@ -181,6 +196,8 @@ class GeometryModel(object):
         logger.info(f'Number faces: {self.faces.__len__()}')
         logger.info(f'Number volumes: {self.volumes.__len__()}\n\n')
 
+        self.loaded = True
+
     def get_geo_components(self, geo):
         """
         Get the simultan components linked to the geometric instance
@@ -198,57 +215,95 @@ class GeometryModel(object):
         """
         return self.template_parser.get_py_geo_components(geo, template_name=template_name)
 
+    def create_layer(self, name, layer=None, **kwags):
+        new_layer = self.LayerCls.create_new(self._wrapped_obj, name, layer=layer, **kwags)
+        if self._layers is None:
+            self._layers = []
+        return new_layer
 
-# def create_geo_classes(geo_types):
-#     """
-#     Create new classes from geometric base classes
-#
-#     :return:
-#     """
-#
-#     logger.debug('creating base geo classes')
-#
-#     class GeoBaseClass(geo_types['base']):
-#         pass
-#
-#     class GeometricLayer(geo_types['layer']):
-#         pass
-#
-#     class GeometricVertex(geo_types['vertex']):
-#         pass
-#
-#     class GeometricEdge(geo_types['edge']):
-#         def get_vertices(self):
-#             return [GeometricVertex.get_obj_by_id(x.Id) for x in self._wrapped_obj.Vertices.Items]
-#
-#     class GeometricEdgeLoop(geo_types['edge_loop']):
-#         def get_edges(self):
-#             return [GeometricEdge.get_obj_by_id(x.Edge.Id) for x in self._wrapped_obj.Edges.Items]
-#
-#     class GeometricFace(geo_types['face']):
-#
-#         @property
-#         def boundary(self):
-#             return GeometricEdgeLoop.get_obj_by_id(self._wrapped_obj.Boundary.Id)
-#
-#         @property
-#         def holes(self):
-#             return [GeometricEdgeLoop.get_obj_by_id(x.Id) for x in self._wrapped_obj.Holes.Items]
-#
-#         @property
-#         def points(self):
-#             return self.boundary.points
-#
-#     class GeometricVolume(geo_types['volume']):
-#
-#         @property
-#         def faces(self):
-#             if self._faces is None:
-#                 self._faces = [GeometricFace.get_obj_by_id(x.Face.Id) for x in self._wrapped_obj.Faces.Items]
-#             return self._faces
-#
-#         @faces.setter
-#         def faces(self, value):
-#             self._faces = value
-#
-#     return GeoBaseClass, GeometricLayer, GeometricVertex, GeometricEdge, GeometricEdgeLoop, GeometricFace, GeometricVolume
+    def create_vertex(self, position, layer=None, **kwags):
+        """
+        Creates a new vertex in this geometry model
+        :param position: np.array, tuple or list with three float; example: [0, 1, 12.25], np.array([0, 1, 12.25]), (0, 1, 12.25)
+        :param layer: Instance of PySimultan.geo_default_types.BaseGeometricLayer
+        :return: Instance of PySimultan.geo_default_types.BaseGeometricVertex
+        """
+        new_vertex = self.VertexCls.create_new(self._wrapped_obj, position, layer, **kwags)
+        if self._vertices is None:
+            self._vertices = []
+        self._vertices.append(new_vertex)
+        return new_vertex
+
+    def create_edge(self, v1, v2, layer=None, **kwags):
+        """
+        Creates a new edge in this geometry model
+        :param v1: Vertex 1; Instance of PySimultan.geo_default_types.BaseGeometricVertex
+        :param v2: Vertex 2; Instance of PySimultan.geo_default_types.BaseGeometricVertex
+        :param layer: Instance of PySimultan.geo_default_types.BaseGeometricLayer
+        :return: Instance of PySimultan.geo_default_types.BaseGeometricEdge
+        """
+        new_edge = self.EdgeCls.create_new(self._wrapped_obj, v1, v2, layer, **kwags)
+        if self._edges is None:
+            self._edges = []
+
+        self._edges.append(new_edge)
+        return new_edge
+
+    def create_edge_loop(self, edges, layer=None, **kwags):
+        """
+        Creates a new edge_loop in this geometry model
+        :param edges: List, tuple or np.array with Instance of PySimultan.geo_default_types.BaseGeometricEdge
+        :param layer: Instance of PySimultan.geo_default_types.BaseGeometricLayer
+        :return: Instance of PySimultan.geo_default_types.BaseGeometriceEdgeLoop
+        """
+        new_edge_loop = self.EdgeLoopCls.create_new(self._wrapped_obj, edges, layer, **kwags)
+        if self._edge_loops is None:
+            self._edge_loops = []
+        self._edge_loops.append(new_edge_loop)
+        return new_edge_loop
+
+    def create_face(self, edge_loop, orientation='forward', openings=None, layer=None, **kwags):
+        """
+        Creates a new face in this geometry model
+        :param edge_loop: Instance of PySimultan.geo_default_types.BaseGeometriceEdgeLoop
+        :param orientation: Orientation of the face; default is 'forward'; Valid values: 'forward', 'backward', 'undefined'
+        GeometricOrientation.Backward, GeometricOrientation.Undefined
+        :param openings: List, array or tuple with PySimultan.geo_default_types.BaseGeometriceEdgeLoop
+        :param layer: Instance of PySimultan.geo_default_types.BaseGeometriceEdgeLoop
+        :return: Instance of PySimultan.geo_default_types.BaseGeometriceFace
+        """
+        # if orientation == 'forward':
+        #     orientation = GeometricOrientation.Forward
+        # elif orientation == 'backward':
+        #     orientation = GeometricOrientation.Backward
+        # else:
+        #     orientation = GeometricOrientation.Undefined
+
+        new_face = self.FaceCls.create_new(self._wrapped_obj, edge_loop, orientation, openings, layer, **kwags)
+        if self._faces is None:
+            self._faces = []
+        self._faces.append(new_face)
+        return new_face
+
+    def create_volume(self, faces, layer=None, **kwags):
+        new_volume = self.VolumeCls.create_new(self._wrapped_obj, faces, layer, **kwags)
+        if self._volumes is None:
+            self._volumes = []
+        self._volumes.append(new_volume)
+        return new_volume
+
+    def associate_face_with_component(self, component, face):
+        """
+        Associate a face with a component
+        :param component: Instance of PySimultan.default_types.SimultanObject, PySimultan.default_types.List, PySimultan.default_types.ReferenceList
+        :param face: Instance of PySimultan.geo_default_types.BaseGeometriceFace
+        """
+        face.associate_with_component(component, exch=self.template_parser.current_data_model.exch)
+
+    def associate_volume_with_component(self, component, volume):
+        """
+
+        :param component:
+        :param volume:
+        """
+        volume.associate_with_component(component, exch=self.template_parser.current_data_model.exch)
